
#include <clutter/clutter.h>
#include <stdlib.h>
#include <string.h>

#define SCN_WIDTH  800
#define SCN_HEIGHT 480

const char *MAIN_STR = "Hello World";
const char *TEXT_FONT = "Courier Bold 170px";
const int FONT_SIZE = 170;
const guint32 colors[] = {0xFF0000FF/*red*/,    0x00FF00FF/*green*/,  
                          0x0000FFFF/*blue*/,   0xFFFF00FF/*yellow*/, 
                          0xFF00FFFF/*pink*/,   0x00FFFFFF/*teal*/,
                          0xFF8000FF/*orange*/, 0xFFFFFFFF/*white*/};
const int NUM_COLORS = 8;
const int NUM_BEHAVIOURS = 3;

// Return the next color in the colors[] array 
ClutterColor *get_next_color ()
{
    static int i=0;
    ClutterColor *cc = malloc (sizeof (ClutterColor));
    clutter_color_from_pixel (cc, colors[i++]);
    if (i > (NUM_COLORS-1)) {
        i=0;
    }
    return cc;
}

gint get_rand_option (gint opt1, gint opt2, gint opt3)
{
    gint num = rand()%3;
    if (num==0) {
        return opt1;
    } else if (num==1) {
        return opt2;
    } else {
        return opt3;
    }
}

// Each time the frame is painted, shade or lighten the coloring
void on_new_frame (ClutterTimeline *tl, 
                   gint frame_num, 
                   gpointer data)
{
    static gboolean get_darker = TRUE;

    ClutterActor *actor = CLUTTER_ACTOR(data);
    ClutterColor *orig_color = g_object_get_data (G_OBJECT (actor), "color");
    gint color_adj = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(actor), 
                                                       "color_adjustment"));

    if (get_darker) {
        color_adj --;
        if (color_adj < -100) {
            color_adj = -100;
            get_darker = FALSE;
        }
    } else {
        color_adj += 1;
        if (color_adj > 0) {
            color_adj = 0;
            get_darker = TRUE;
        }
    }
    gdouble scale = 1.0+(0.005*(gdouble)color_adj);
    ClutterColor c;
    clutter_color_shade (orig_color, &c, scale);
    clutter_label_set_color (CLUTTER_LABEL(actor), &c); 
    g_object_set_data (G_OBJECT (actor), 
                       "color_adjustment", 
                       GINT_TO_POINTER(color_adj));
}

ClutterActor *create_new_character (char c, int xpos, int ypos) 
{
    ClutterActor *actor; 
    ClutterColor *color;
    char sz[2];
    sz[0] = c;
    sz[1] = '\0';

    color = get_next_color(); 
    actor = clutter_label_new_full (TEXT_FONT, sz, color);
    clutter_actor_set_position (actor, xpos, ypos);

    //Setup animation
    ClutterTimeline *timeline;
    timeline = clutter_timeline_new (360, 60);    //num frames, fps
    clutter_timeline_set_loop (timeline, TRUE);

    //Create animation behavior (rotations)
    ClutterAlpha *alpha;
    ClutterBehaviour *behaviour;
    alpha = clutter_alpha_new_full (timeline, 
                                    (rand()%2) ?
                                      CLUTTER_ALPHA_RAMP_DEC :
                                      CLUTTER_ALPHA_RAMP_INC,
                                    NULL, NULL);

    behaviour = clutter_behaviour_rotate_new (alpha, 
                                              get_rand_option(CLUTTER_X_AXIS,
                                                              CLUTTER_Y_AXIS,
                                                              CLUTTER_Z_AXIS),
                                              CLUTTER_ROTATE_CW,
                                              0.0, 360.0);
    clutter_behaviour_rotate_set_center (CLUTTER_BEHAVIOUR_ROTATE (behaviour),
                                         FONT_SIZE/2, FONT_SIZE/2, 0);

    clutter_behaviour_apply (behaviour, actor);
    g_signal_connect (timeline, "new-frame", G_CALLBACK (on_new_frame), actor);
    clutter_timeline_start (timeline);

    //save variables for later destruction
    g_object_set_data (G_OBJECT (actor), "color", color);
    g_object_set_data (G_OBJECT (actor), "color_adjustment", GINT_TO_POINTER(0));
    g_object_set_data (G_OBJECT (actor), "timeline", timeline);
    g_object_set_data (G_OBJECT (actor), "alpha", alpha);
    g_object_set_data (G_OBJECT (actor), "behaviour", behaviour);
    return actor;
}

void clean_up (ClutterActor *ca[], gint num_actors)
{
    ClutterColor *color;
    ClutterTimeline *timeline;
    ClutterAlpha *alpha;
    ClutterBehaviour *behaviour;
    
    gint i=0;
    for (i=0; i<num_actors; i++) {
        color =  (ClutterColor *)g_object_get_data (G_OBJECT (ca[i]), 
                                                    "color");
        if (color) {
            g_free (color);
        }
        timeline =  (ClutterTimeline *)g_object_get_data (G_OBJECT (ca[i]),
                                                          "timeline");
        if (timeline) {
            g_free (timeline);
        }
        alpha =  (ClutterAlpha *)g_object_get_data (G_OBJECT (ca[i]),
                                                    "alpha");
        if (alpha) {
            g_free (alpha);
        }
        behaviour =  (ClutterBehaviour *)g_object_get_data (G_OBJECT (ca[i]), 
                                                            "behaviour");
        if (behaviour) {
            g_free (behaviour);
        }
    }
}

/*
// ret FALSE to close, TRUE to prevent
static gboolean query_close ( GtkWidget *widget, GdkEvent  *event,
                              gpointer   data )
{
    g_print ("delete event occurred\n");
    return FALSE;
}

static void close( GtkWidget *widget, gpointer data )
{
    clutter_main_quit ();
}
*/

int main (int argc, char **argv)
{
    const ClutterColor scn_bkgd = {0x00,0x00,0xFF,0xFF};
    ClutterActor *stage;

    srand(time(NULL)); //for random colors

    clutter_init(&argc, &argv);

    stage = clutter_stage_get_default();

/*    g_signal_connect (G_OBJECT (stage), "delete_event",
		      G_CALLBACK (query_close), NULL);
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (close), NULL);
*/
    gint scn_width = SCN_WIDTH, scn_height = SCN_HEIGHT;
    clutter_actor_set_size (stage, scn_width, scn_height);
    clutter_stage_set_color (CLUTTER_STAGE(stage), &scn_bkgd);
    
    //show MAIN_STR
    gint num_actors = 0;
    ClutterActor *actor;
    ClutterActor *arrActors[strlen(MAIN_STR)];
    gint xpos = 0;
    int i=0;
    for (i=0; i<strlen(MAIN_STR); i++) {
        xpos += FONT_SIZE/3;
        if (MAIN_STR[i]==' ') {
            continue;
        }
        actor = create_new_character (MAIN_STR[i], xpos, (scn_height-FONT_SIZE)/2);
        arrActors[num_actors++] = actor;
        clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);
    }

    clutter_actor_show_all (stage);
    clutter_main ();

    return EXIT_SUCCESS;

}
