#include <clutter/clutter.h>

#define LOWER_BOUND 50
#define UPPER_BOUND 255

ClutterActor *stage, *black_square, *red_square_1, *red_square_2, 
                     *red_square_3, *red_square_4;
ClutterActor *start_button, *start_shadow, *start_label;

ClutterTimeline *button_press_timeline, *button_release_timeline, 
                *growth_timeline, *button_reshow_timeline;

ClutterEffectTemplate *button_press_movement, *button_release_movement;

gint rotation_angle;
int square_size = LOWER_BOUND;

/**
  Rotate the black square 360 degrees.
  */
void rotate_square()
{
  rotation_angle += 1;
  if (rotation_angle >= 360)
      rotation_angle = 0;

  clutter_actor_set_rotation(black_square, 
                             CLUTTER_Z_AXIS, 
                             rotation_angle, 
                             113, 113, 5);
  return;
}


/**
  The black square appears to grow because the
  four red squares grow in proportional width/
  height until there is only a 1 pixel width
  where the viewer can still see the rotating
  black square behind the 4 red ones.
  */
void start_square_growing(ClutterTimeline *timeline, gint frame_num, 
                          gpointer data)
{

  if (square_size == UPPER_BOUND){
     clutter_timeline_stop(growth_timeline);
     return;
  }
  square_size++;
  clutter_actor_set_size(red_square_1, square_size, square_size);

  gint curr_x_pos = 0;
  gint curr_y_pos = 0;
  gint new_anchor = 0;

  guint curr_width = clutter_actor_get_width(red_square_2);
  clutter_actor_get_position(red_square_2, &curr_x_pos, &curr_y_pos);
  gint change_in_anchor = square_size - curr_width;

  //change square 2
  if (square_size > curr_width)
  {
      new_anchor = curr_x_pos - change_in_anchor;
  }
  else if (square_size < curr_width)
  {
      new_anchor = curr_x_pos + change_in_anchor;
  }
  clutter_actor_set_x(red_square_2, new_anchor);
  clutter_actor_set_size(red_square_2, square_size, square_size);

  
  //change square 3
  clutter_actor_get_position(red_square_3, &curr_x_pos, &curr_y_pos);
  if (square_size > curr_width)
  {
      new_anchor = curr_y_pos - change_in_anchor;
  }  
  else if (square_size < curr_width)
  {
      new_anchor = curr_y_pos + change_in_anchor;
  }
  clutter_actor_set_y(red_square_3, new_anchor);
  clutter_actor_set_size(red_square_3, square_size, square_size);

  
  //change square 4
  clutter_actor_get_position(red_square_4, &curr_x_pos, &curr_y_pos);
  if (square_size > curr_width)
  {
      new_anchor = curr_y_pos - change_in_anchor;
  }
  else if (square_size < curr_width)
  {
      new_anchor = curr_y_pos + change_in_anchor;
  }
  clutter_actor_set_y(red_square_4, new_anchor);
  if (square_size > curr_width)
  {
      new_anchor = curr_x_pos - change_in_anchor;
  }
  else if (square_size < curr_width)
  {
      new_anchor = curr_x_pos - change_in_anchor;
  }
  clutter_actor_set_x(red_square_4, new_anchor);
  clutter_actor_set_size(red_square_4, square_size, square_size);

    if (square_size == UPPER_BOUND - 1){
     printf("we get to reseting square_size\n");
     square_size = LOWER_BOUND;
     return;
  }
  return;

}



/**
  When the cursor enters the start button, change
  the start button's color.
  */
gboolean on_button_enter(ClutterActor *actor, ClutterEvent *event, 
                         gpointer data)
{
  ClutterColor color_red   = { 0xff, 0x00, 0x00, 0xff };
  clutter_rectangle_set_color(CLUTTER_RECTANGLE(start_button), &color_red);

  return TRUE;
}



/**
  When the cursor leaves the start button, change
  the button color back to it's original color.
  */
gboolean on_button_leave(ClutterActor *actor, ClutterEvent *event, 
                         gpointer data)
{
  ClutterColor color_white = { 0xff, 0xff, 0xff, 0xff };
  clutter_rectangle_set_color(CLUTTER_RECTANGLE(start_button), &color_white);

  return TRUE;
}



/**
  When the mouse is pressed over the start button, check if the 
  red square sizes are larger than certain amount - if so, the 
  simulation needs to be reset.
  Then move the start button and text southwest to simulate a button
  press.
  */
gboolean on_button_press(ClutterActor *actor, ClutterEvent *event, 
                         gpointer data)
{
  if (square_size >= 75){
    square_size = LOWER_BOUND;

    clutter_actor_set_size(red_square_1, square_size, square_size);
    clutter_actor_set_size(red_square_2, square_size, square_size);
    clutter_actor_set_size(red_square_3, square_size, square_size);
    clutter_actor_set_size(red_square_4, square_size, square_size);
    clutter_actor_set_position(red_square_1, 509, 74);
    clutter_actor_set_position(red_square_2, 910, 74);
    clutter_actor_set_position(red_square_3, 509, 475);
    clutter_actor_set_position(red_square_4, 910, 475);
  }
  ClutterColor color_red   = { 0xff, 0x00, 0x00, 0xff };
  clutter_rectangle_set_color(CLUTTER_RECTANGLE(start_button), &color_red);
  button_press_timeline = clutter_effect_move(button_press_movement, 
                                              actor, 200, 455, NULL, NULL);
  button_press_timeline = clutter_effect_move(button_press_movement, 
                                              start_label, 227, 465, NULL, NULL);

  return TRUE;
}



/**
  Delays the hiding of the button. (So that the
  viewer doesn't miss the button coming back
  up to its original position after being pressed.)
  */
void hide_button(ClutterActor *actor, gpointer data)
{
  clutter_timeline_set_delay(button_release_timeline, 2000);
  clutter_timeline_pause(button_release_timeline);
  clutter_timeline_start(button_release_timeline);

  clutter_actor_hide(actor);
  clutter_actor_hide(start_label);
  clutter_actor_hide(start_shadow);  
  return;
}



/**
  When the mouse is released over the start button,
  the simulation begins. The red squares grow to 
  a maximum width and height before stopping.
  */
gboolean on_button_release(ClutterActor *actor, ClutterEvent *event, 
                           gpointer data)
{
  ClutterColor color_white = { 0xff, 0xff, 0xff, 0xff };
  clutter_rectangle_set_color(CLUTTER_RECTANGLE(start_button), &color_white);

  growth_timeline = clutter_timeline_new(175, 20);
  g_signal_connect(growth_timeline, "new-frame", 
                   G_CALLBACK(start_square_growing), NULL);
  clutter_timeline_start(growth_timeline);

  button_release_timeline = clutter_effect_move(button_release_movement, 
                                      actor, 205, 450, 
                                      (ClutterEffectCompleteFunc)hide_button, 
                                      NULL);
  button_release_timeline = clutter_effect_move(button_release_movement, 
                                      start_label, 232, 460, NULL, NULL);
  clutter_timeline_start(button_reshow_timeline);

  return TRUE;
}



/**
  Show the button. (This is called when the simulation
  has completed. This was done because if the start
  button is pressed during the simulation, it has a 
  tendency to skew the red squares to incorrect
  positions.)
  */
void show_button(ClutterTimeline *timeline, gpointer data)
{
  clutter_actor_show(start_button);
  clutter_actor_show(start_label);
  clutter_actor_show(start_shadow);  
  return;
}



int main(int argc, char *argv[])
{
  clutter_init(&argc, &argv);

  stage = clutter_stage_get_default();
  clutter_actor_set_size(stage, 1024, 600);
  ClutterColor stage_color = { 0xff, 0xff, 0xff, 0xff }; 
  ClutterColor tab_color   = { 0xcd, 0xc1, 0xc5, 0xff };
  ClutterColor color_white = { 0xff, 0xff, 0xff, 0xff };
  ClutterColor color_black = { 0x00, 0x00, 0x00, 0xff };
  ClutterColor color_red   = { 0xff, 0x00, 0x00, 0xff };
  clutter_stage_set_color(CLUTTER_STAGE(stage), &stage_color);
  clutter_stage_set_title(CLUTTER_STAGE(stage), "Squares");
  clutter_actor_show(stage);

  /* Tab */
  ClutterActor *tab = clutter_rectangle_new_with_color(&tab_color);
  clutter_actor_set_size             (tab, 1000, 576);
  clutter_actor_set_position         (tab, 10, 10);
  clutter_rectangle_set_border_width (CLUTTER_RECTANGLE(tab), 1);
  clutter_rectangle_set_border_color (CLUTTER_RECTANGLE(tab), &color_black);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), tab);
  clutter_actor_show                 (tab);
  
  /* Border square shadow */
  ClutterActor *border_shadow = clutter_rectangle_new_with_color(&color_black);
  clutter_actor_set_opacity          (border_shadow, 255);
  clutter_actor_set_size             (border_shadow, 455, 455);
  clutter_actor_set_position         (border_shadow, 502, 77);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), border_shadow);
  clutter_actor_show                 (border_shadow);

  /* Border square */
  ClutterActor *border_square = clutter_rectangle_new_with_color(&color_white);
  clutter_actor_set_size             (border_square, 455, 455);
  clutter_actor_set_position         (border_square, 507, 72);
  clutter_rectangle_set_border_color (CLUTTER_RECTANGLE(border_square), 
                                      &color_black);
  clutter_rectangle_set_border_width (CLUTTER_RECTANGLE(border_square), 2);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), border_square);
  clutter_actor_show                 (border_square);

  /* Black square */
  black_square = clutter_rectangle_new_with_color(&color_black);
  clutter_actor_set_size             (black_square, 225, 225);
  clutter_actor_set_position         (black_square, 622, 187);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), black_square);
  clutter_actor_show                 (black_square);
  
  /* North-west red square */
  red_square_1 = clutter_rectangle_new_with_color(&color_red);
  clutter_actor_set_size             (red_square_1, 50, 50);
  clutter_actor_set_position         (red_square_1, 509, 74);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), red_square_1);
  clutter_actor_show                 (red_square_1);

  /* North-east red square */
  red_square_2 = clutter_rectangle_new_with_color(&color_red);
  clutter_actor_set_size             (red_square_2, 50, 50);
  clutter_actor_set_position         (red_square_2, 910, 74);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), red_square_2);
  clutter_actor_show                 (red_square_2);

  /* South-west red square */
  red_square_3 = clutter_rectangle_new_with_color(&color_red);
  clutter_actor_set_size             (red_square_3, 50, 50);
  clutter_actor_set_position         (red_square_3, 509, 475);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), red_square_3);
  clutter_actor_show                 (red_square_3);

  /* South-east red square */
  red_square_4 = clutter_rectangle_new_with_color(&color_red);
  clutter_actor_set_size             (red_square_4, 50, 50);
  clutter_actor_set_position         (red_square_4, 910, 475);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), red_square_4);
  clutter_actor_show                 (red_square_4);
 
  /* Rotation timeline for black square */
  ClutterTimeline *rotation_timeline = clutter_timeline_new(50, 50);
  button_reshow_timeline = clutter_timeline_new_for_duration(10000);
  g_signal_connect(rotation_timeline, "new-frame", G_CALLBACK(rotate_square), 
                   NULL);
  g_signal_connect(button_reshow_timeline, "completed", G_CALLBACK(show_button), 
                   NULL);
  clutter_timeline_set_loop(rotation_timeline, TRUE);
  clutter_timeline_start(rotation_timeline);

  /* start shadow */
  start_shadow = clutter_rectangle_new_with_color(&color_black);
  clutter_actor_set_opacity          (start_shadow, 255);
  clutter_actor_set_size             (start_shadow, 100, 50);
  clutter_actor_set_position         (start_shadow, 200, 455);
  clutter_container_add_actor        (CLUTTER_CONTAINER(stage), start_shadow);
  clutter_actor_show                 (start_shadow);

  /* Add start button */
  ClutterColor button_color = { 0xb0, 0x17, 0x1f, 0xff };
  start_button = clutter_rectangle_new_with_color(&color_white);
  clutter_actor_set_size(start_button, 100, 50);
  clutter_actor_set_position(start_button, 205, 450);
  clutter_rectangle_set_border_color (CLUTTER_RECTANGLE(start_button), 
                                      &color_black);
  clutter_rectangle_set_border_width (CLUTTER_RECTANGLE(start_button), 1);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), start_button);

  button_press_timeline = clutter_timeline_new(2, 30);
  button_press_movement = clutter_effect_template_new(button_press_timeline, 
                          CLUTTER_ALPHA_RAMP_INC);

  button_release_timeline = clutter_timeline_new(2, 3);
  button_release_movement = clutter_effect_template_new(
                            button_release_timeline, CLUTTER_ALPHA_RAMP_INC);

  clutter_actor_set_reactive(start_button, TRUE);
  g_signal_connect(start_button, "enter-event", G_CALLBACK(on_button_enter), 
                   NULL);
  g_signal_connect(start_button, "leave-event", G_CALLBACK(on_button_leave), 
                   NULL);
  g_signal_connect(start_button, "button-release-event", 
                   G_CALLBACK(on_button_release), NULL);
  g_signal_connect(start_button, "button-press-event", 
                   G_CALLBACK(on_button_press), NULL);
 
  /* button text */
  start_label =  clutter_label_new_full("Sans 14", "Start", &color_black);
  clutter_actor_set_position(start_label, 232, 460);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), start_label);
  clutter_actor_show(start_label);

  /* Title text */
  ClutterActor *title_label = clutter_label_new_full("Sans 20", 
                                                     "Squares", &color_black);
  clutter_actor_set_position(title_label, 40, 30);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), title_label);
  clutter_actor_show(title_label);

  /* Title text shadow */
  ClutterActor *title_shadow = clutter_label_new_full("Sans 20", 
                                                      "Squares", &color_black);
  clutter_actor_set_position(title_shadow, 38, 32);
  clutter_actor_set_opacity(title_shadow, 70);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), title_shadow);
  clutter_actor_show(title_shadow);
  
  /* Description text */
  ClutterActor *description_line_1 = clutter_label_new_full("Sans 16",
	"The rotating black square will never", &color_black);
  clutter_actor_set_position(description_line_1, 50, 100);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), description_line_1);
  clutter_actor_show(description_line_1);
  
  ClutterActor *description_line_2 = clutter_label_new_full("Sans 16",
	"change its size. Watch to see how the", &color_black);
  clutter_actor_set_position(description_line_2, 50, 150);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), description_line_2);
  clutter_actor_show(description_line_2);

  ClutterActor *description_line_3 = clutter_label_new_full("Sans 16",
	"black square appears to grow...", &color_black);
  clutter_actor_set_position(description_line_3, 50, 200);
  clutter_container_add_actor(CLUTTER_CONTAINER(stage), description_line_3);
  clutter_actor_show(description_line_3);

  clutter_main();

  g_object_unref(rotation_timeline);
  g_object_unref(growth_timeline);
  g_object_unref(button_reshow_timeline);
  return 0;
}
